@@ -9,6 +9,7 @@
  */
 
 #include "grib_api_internal.h"
+#include "eccodes.h"
 
 #define NUMBER(x) (sizeof(x) / sizeof(x[0]))
 
@@ -40,12 +41,23 @@ static void check_float_representation(const double val, const double expected,
     compare_doubles(out, expected, tolerance);
 }
 
+static void test_get_package_name()
+{
+    printf("Running %s ...\n", __func__);
+
+    const char* pn = codes_get_package_name();
+    Assert(pn != NULL);
+    Assert(strlen(pn) > 1);
+    printf("Package name = %s\n", pn);
+}
+
 static void test_get_git_sha1()
 {
     printf("Running %s ...\n", __func__);
 
-    const char* sha1 = grib_get_git_sha1();
+    const char* sha1 = codes_get_git_sha1();
     Assert(sha1 != NULL);
+    printf("Git SHA1 = %s\n", sha1);
 }
 
 static void test_get_build_date()
@@ -91,7 +103,7 @@ static void test_gaussian_latitudes(int order)
     double lat1 = 0, lat2 = 0;
     double* lats = (double*)malloc(sizeof(double) * num);
 
-    ret = grib_get_gaussian_latitudes(order, lats);
+    ret = codes_get_gaussian_latitudes(order, lats);
     Assert(ret == GRIB_SUCCESS);
 
     lat1 = lats[0];
@@ -112,7 +124,7 @@ static void test_gaussian_latitude_640()
     int ret                = 0;
     const double tolerance = 1e-6;
     double* lats           = (double*)malloc(sizeof(double) * num);
-    ret                    = grib_get_gaussian_latitudes(order, lats);
+    ret                    = codes_get_gaussian_latitudes(order, lats);
     Assert(ret == GRIB_SUCCESS);
 
     compare_doubles(lats[0], 89.892396, tolerance);
@@ -212,8 +224,7 @@ static void test_assertion_catching()
     printf("Running %s ...\n", __func__);
 
     char empty[] = "";
-    char** list  = 0;
-    int i        = 0;
+    char** list  = NULL;
     Assert(assertion_caught == 0);
     codes_set_codes_assertion_failed_proc(&my_assertion_proc);
 
@@ -221,13 +232,14 @@ static void test_assertion_catching()
     list = string_split(empty, " ");
 
     Assert(assertion_caught == 1);
+    Assert( list == NULL );
 
     /* Restore everything */
     codes_set_codes_assertion_failed_proc(NULL);
     assertion_caught = 0;
 
-    for (i = 0; list[i] != NULL; ++i)
-        free(list[i]);
+    // for (i = 0; list[i] != NULL; ++i)
+    //     free(list[i]);
     free(list);
 }
 
@@ -252,6 +264,16 @@ static void test_logging_proc()
     Assert(logging_caught == 0);
 }
 
+static void my_print_proc(const grib_context* c, void* descriptor, const char* mesg)
+{
+}
+static void test_print_proc()
+{
+    grib_context* context = grib_context_get_default();
+    grib_context_set_print_proc(context, my_print_proc);
+    grib_context_set_print_proc(context, NULL);
+}
+
 static void test_concept_condition_strings()
 {
     printf("Running %s ...\n", __func__);
@@ -260,6 +282,7 @@ static void test_concept_condition_strings()
     char result[1024] = {0,};
     grib_context* context = NULL;
     grib_handle* h = grib_handle_new_from_samples(context, "GRIB2");
+    if (!h) return;
 
     err = get_concept_condition_string(h, "typeOfLevel", NULL, result);
     Assert(!err);
@@ -378,9 +401,44 @@ static void test_gribex_mode()
 
     Assert( grib_get_gribex_mode(c) == 0 ); /* default is OFF */
     grib_gribex_mode_on(c);
+    codes_gribex_mode_on(c);
     Assert( grib_get_gribex_mode(c) == 1 );
     grib_gribex_mode_off(c);
+    codes_gribex_mode_off(c);
     Assert( grib_get_gribex_mode(c) == 0 );
+    Assert( codes_get_gribex_mode(c) == 0 );
+}
+
+static void test_gts_header_mode()
+{
+    grib_context* c = grib_context_get_default();
+    printf("Running %s ...\n", __func__);
+
+    grib_gts_header_on(c);
+    codes_gts_header_on(c);
+    Assert(c->gts_header_on == 1);
+    grib_gts_header_off(c);
+    codes_gts_header_off(c);
+    Assert(c->gts_header_on == 0);
+}
+
+static void test_data_quality_checks()
+{
+    grib_context* c = grib_context_get_default();
+    printf("Running %s ...\n", __func__);
+
+    grib_context_set_data_quality_checks(c, 1);//warning
+    grib_context_set_data_quality_checks(c, 2);//error
+    grib_context_set_data_quality_checks(c, 0);//no checks
+}
+
+static void test_bufr_multi_element_constant_arrays()
+{
+    grib_context* c = grib_context_get_default();
+    printf("Running %s ...\n", __func__);
+
+    codes_bufr_multi_element_constant_arrays_on(c);
+    codes_bufr_multi_element_constant_arrays_off(c);
 }
 
 static void test_grib_binary_search()
@@ -428,7 +486,7 @@ static void test_parse_keyval_string()
                               values_required, GRIB_TYPE_UNDEFINED, values1, &count);
     Assert( !err );
     Assert( count == 2 );
-    grib_print_values("print values test: values1", values1);
+    grib_print_values("print values test: values1", values1, stdout, count);
 
     Assert( strcmp(values1[0].name, "key1")==0 );
     Assert( strcmp(values1[0].string_value, "value1")==0 );
@@ -448,7 +506,7 @@ static void test_parse_keyval_string()
                               values_required, GRIB_TYPE_LONG, values2, &count);
     Assert( !err );
     Assert( count == 1 );
-    grib_print_values("print values test: values2", values2);
+    grib_print_values("print values test: values2", values2, stdout, count);
     Assert( strcmp(values2[0].name, "x")==0 );
     Assert( values2[0].long_value == 14 );
     Assert( values2[0].equal == 1 );
@@ -459,11 +517,18 @@ static void test_parse_keyval_string()
                               values_required, GRIB_TYPE_DOUBLE, values3, &count);
     Assert( !err );
     Assert( count == 1 );
-    grib_print_values("print values test: values3", values3);
+    grib_print_values("print values test: values3", values3, stdout, count);
     Assert( strcmp(values3[0].name, "mars.level")==0 );
     free( (void*)values3[0].name );
 }
 
+static void test_time_conversions()
+{
+    printf("Running %s ...\n", __func__);
+    long result = convert_to_minutes(120, 13); // 120s = 2mins
+    Assert(result == 2);
+}
+
 static void test_dates()
 {
     printf("Running %s ...\n", __func__);
@@ -546,60 +611,69 @@ void test_scale_factor_scaled_values()
 void test_iarray()
 {
     printf("Running %s ...\n", __func__);
-    grib_context* c = grib_context_get_default();
-    grib_iarray* a = grib_iarray_new(c, 10, 10);
+
+    grib_iarray* a = grib_iarray_new(10, 10);
     grib_iarray_push(a, 42);
     grib_iarray_push(a, 10000);
     grib_iarray_print("iarray", a);
 
-    grib_iarray* b = grib_iarray_new(c, 1, 1);
+    grib_iarray* b = grib_iarray_new(1, 1);
     grib_iarray_push(b, 0);
     grib_iarray_push(b, -1);
     grib_iarray_push(b, +1);
 
-    grib_viarray* va = grib_viarray_new(c, 1, 1);
-    grib_viarray_push(c, va, a);
-    grib_viarray_push(c, va, b);
+    grib_viarray* va = grib_viarray_new(1, 1);
+    grib_viarray_push(va, a);
+    grib_viarray_push(va, b);
     grib_viarray_print("viarray", va);
 
     grib_iarray_delete(a);
     grib_iarray_delete(b);
-    grib_viarray_delete(c, va);
+    grib_viarray_delete(va);
 }
 
 void test_darray()
 {
     printf("Running %s ...\n", __func__);
-    grib_context* c = grib_context_get_default();
-    grib_darray* a = grib_darray_new(c, 10, 10);
-    grib_darray_push(c, a, 42.009);
-    grib_darray_push(c, a, -1.11);
-    grib_darray_push(c, a, 5099);
+
+    grib_darray* a = grib_darray_new(10, 10);
+    grib_darray_push(a, 42.009);
+    grib_darray_push(a, -1.11);
+    grib_darray_push(a, 5099);
     grib_darray_print("darray", a);
 
-    grib_darray* b = grib_darray_new(c, 5, 1);
-    grib_darray_push(c, b, 8);
-    grib_darray_push(c, b, 12);
+    grib_darray* b = grib_darray_new(5, 1);
+    grib_darray_push(b, 8);
+    grib_darray_push(b, 12);
 
-    grib_vdarray* va = grib_vdarray_new(c, 1, 1);
-    grib_vdarray_push(c, va, a);
-    grib_vdarray_push(c, va, b);
+    grib_vdarray* va = grib_vdarray_new(1, 1);
+    grib_vdarray_push(va, a);
+    grib_vdarray_push(va, b);
     grib_vdarray_print("vdarray", va);
 
-    grib_darray_delete(c, a);
-    grib_darray_delete(c, b);
-    grib_vdarray_delete(c, va);
+    grib_darray_delete(a);
+    grib_darray_delete(b);
+    grib_vdarray_delete(va);
 }
 
 void test_sarray()
 {
     printf("Running %s ...\n", __func__);
-    grib_context* c = grib_context_get_default();
-    grib_sarray* a = grib_sarray_new(c, 10, 10);
-    grib_sarray_push(c, a, "ants");
-    grib_sarray_push(c, a, "bugs");
+
+    grib_sarray* a = grib_sarray_new(10, 10);
+
+    char ants_s[] = "ants";
+    char bugs_s[] = "bugs";
+    grib_sarray_push(a, ants_s);
+    grib_sarray_push(a, bugs_s);
     grib_sarray_print("sarray", a);
-    grib_sarray_delete(c, a);
+
+    grib_vsarray* va = grib_vsarray_new(1, 1);
+    grib_vsarray_push(va, a);
+    grib_vsarray_print("vsarray", va);
+
+    grib_sarray_delete(a);
+    grib_vsarray_delete(va);
 }
 
 void test_codes_get_product_name()
@@ -610,12 +684,13 @@ void test_codes_get_product_name()
     Assert( STR_EQUAL("BUFR",  codes_get_product_name(PRODUCT_BUFR)) );
     Assert( STR_EQUAL("GTS",   codes_get_product_name(PRODUCT_GTS)) );
     Assert( STR_EQUAL("METAR", codes_get_product_name(PRODUCT_METAR)) );
+    Assert( STR_EQUAL("TAF",   codes_get_product_name(PRODUCT_TAF)) );
 }
 
 void test_codes_get_type_name()
 {
     printf("Running %s ...\n", __func__);
-    Assert( STR_EQUAL("long",    grib_get_type_name(GRIB_TYPE_LONG)) );
+    Assert( STR_EQUAL("long",    codes_get_type_name(GRIB_TYPE_LONG)) );
     Assert( STR_EQUAL("string",  grib_get_type_name(GRIB_TYPE_STRING)) );
     Assert( STR_EQUAL("double",  grib_get_type_name(GRIB_TYPE_DOUBLE)) );
     Assert( STR_EQUAL("bytes",   grib_get_type_name(GRIB_TYPE_BYTES)) );
@@ -623,9 +698,179 @@ void test_codes_get_type_name()
     Assert( STR_EQUAL("section", grib_get_type_name(GRIB_TYPE_SECTION)) );
 }
 
+void test_grib2_choose_PDTN()
+{
+    printf("Running %s ...\n", __func__);
+    int det = true;
+    int instant = true;
+
+    Assert( 0  == grib2_choose_PDTN(0,  det,  instant) );
+    Assert( 8  == grib2_choose_PDTN(0,  det, !instant) );
+    Assert( 1  == grib2_choose_PDTN(0, !det,  instant) );
+    Assert( 11 == grib2_choose_PDTN(0, !det, !instant) );
+}
+
+void test_grib2_select_PDTN()
+{
+    printf("Running %s ...\n", __func__);
+    int eps = 1;
+    int instant = 1;
+    int chemical = 1;
+    int chemical_srcsink = 1;
+    int chemical_distfn = 1;
+    int aerosol = 1;
+    int aerosol_optical = 1;
+
+    // arguments = eps instant chemical chemical_srcsink chemical_distfn aerosol aerosol_optical
+
+    // Chemicals
+    Assert( 40 == grib2_select_PDTN(!eps, instant,  chemical, 0, 0, 0, 0) );
+    Assert( 41 == grib2_select_PDTN(eps,  instant,  chemical, 0, 0, 0, 0) );
+    Assert( 42 == grib2_select_PDTN(!eps, !instant, chemical, 0, 0, 0, 0) );
+    Assert( 43 == grib2_select_PDTN(eps,  !instant, chemical, 0, 0, 0, 0) );
+
+    // Chemical source/sink
+    Assert( 76 == grib2_select_PDTN(!eps, instant,  !chemical, chemical_srcsink, 0,0,0) );
+    Assert( 77 == grib2_select_PDTN(eps,  instant,  !chemical, chemical_srcsink, 0,0,0) );
+    Assert( 78 == grib2_select_PDTN(!eps, !instant, !chemical, chemical_srcsink, 0,0,0) );
+    Assert( 79 == grib2_select_PDTN(eps,  !instant, !chemical, chemical_srcsink, 0,0,0) );
+
+    // Chemical distrib. function
+    Assert( 58 == grib2_select_PDTN(eps, instant,  0, 0, chemical_distfn, 0, 0) );
+    Assert( 68 == grib2_select_PDTN(eps, !instant,  0, 0, chemical_distfn, 0, 0) );
+    Assert( 57 == grib2_select_PDTN(!eps, instant,  0, 0, chemical_distfn, 0, 0) );
+    Assert( 67 == grib2_select_PDTN(!eps, !instant,  0, 0, chemical_distfn, 0, 0) );
+
+    // Aerosols
+    Assert( 48 == grib2_select_PDTN(!eps, instant,  0, 0, 0, aerosol, 0) );
+    Assert( 46 == grib2_select_PDTN(!eps, !instant, 0, 0, 0, aerosol, 0) );
+    Assert( 45 == grib2_select_PDTN(eps, instant,   0, 0, 0, aerosol, 0) );
+    Assert( 85 == grib2_select_PDTN(eps, !instant,  0, 0, 0, aerosol, 0) );
+
+    // Aerosol optical
+    Assert( 49 == grib2_select_PDTN(eps, instant,  0, 0, 0, 0, aerosol_optical) );
+    Assert( 48 == grib2_select_PDTN(!eps, instant, 0, 0, 0, 0, aerosol_optical) );
+
+    // Plain vanilla
+    Assert(  0 == grib2_select_PDTN(!eps, instant,  !chemical, !chemical_srcsink, !chemical_distfn, !aerosol, 0) );
+    Assert(  1 == grib2_select_PDTN(1,1,0,0,0, !aerosol,0) );
+    Assert(  8 == grib2_select_PDTN(0,0,0,0,0, !aerosol,0) );
+    Assert( 11 == grib2_select_PDTN(1,0,0,0,0, !aerosol,0) );
+
+    //printf("%d\n", grib2_select_PDTN(!eps, instant,  !chemical, !chemical_srcsink, !chemical_distfn, aerosol, 0) );
+    Assert( 1 == grib2_is_PDTN_EPS(1) );
+    Assert( 1 == grib2_is_PDTN_EPS(11) );
+    Assert( 0 == grib2_is_PDTN_EPS(0) );
+}
+
+void test_codes_get_error_message()
+{
+    printf("Running %s ...\n", __func__);
+    const char* errmsg = grib_get_error_message(6666);
+    Assert( STR_EQUAL(errmsg, "Unknown error -6666"));
+}
+
+void test_codes_context_set_debug()
+{
+    printf("Running %s ...\n", __func__);
+    grib_context* context = NULL;
+    int err = 0;
+
+    printf("\tEnable debugging...\n");
+    grib_context_set_debug(context, -1);
+
+    grib_handle* h = grib_handle_new_from_samples(context, "GRIB2");
+    if (!h) return;
+    err = grib_set_long(h, "paramId", 167);
+    Assert(!err);
+
+    printf("\tDisable debugging...\n");
+    grib_context_set_debug(context, 0);
+
+    err = grib_set_long(h, "edition", 1);
+    Assert(!err);
+    printf("\tEnable debugging again (verbose)...\n");
+    grib_context_set_debug(context, 1);
+    grib_handle_delete(h);
+
+    grib_context_set_debug(context, 0);
+}
+
+void test_codes_is_feature_enabled()
+{
+    printf("Running %s ...\n", __func__);
+    const char* features[] = {
+        "AEC",
+        "MEMFS",
+        "JPG",
+        "PNG",
+        "ECCODES_THREADS",
+        "ECCODES_OMP_THREADS",
+        "NETCDF",
+        "FORTRAN",
+        "GEOGRAPHY",
+        NULL};
+    for (int i = 0; features[i]; ++i) {
+        const char* f = features[i];
+        printf("\tFeature %s enabled?\t%d\n", f, codes_is_feature_enabled(f));
+    }
+    Assert( 0 == codes_is_feature_enabled("total rubbish") );
+}
+
+void test_codes_get_features()
+{
+    printf("Running %s ...\n", __func__);
+
+    size_t len = 512;
+    char* features = (char*)calloc(len, sizeof(char));
+    int err = codes_get_features(features, &len, CODES_FEATURES_ALL);
+    Assert(!err);
+    printf("\tFeatures are: '%s'\n", features);
+
+    len = 512;
+    err = codes_get_features(features, &len, CODES_FEATURES_ENABLED);
+    printf("\tEnabled features are: '%s'\n", features);
+
+    len = 512;
+    err = codes_get_features(features, &len, CODES_FEATURES_DISABLED);
+    printf("\tDisabled features are: '%s'\n", features);
+
+    free(features);
+}
+
+static void test_grib_get_binary_scale_fact()
+{
+    printf("Running %s ...\n", __func__);
+    int err = 0;
+    long result = grib_get_binary_scale_fact(INFINITY, 0, 0, &err);
+    Assert( err == GRIB_OUT_OF_RANGE);
+    Assert( result == 0 );
+
+    result = grib_get_binary_scale_fact(100, 0, 65, &err); // bpv too big
+    Assert( err == GRIB_OUT_OF_RANGE);
+    Assert( result == 0 );
+
+    result = grib_get_binary_scale_fact(100, 0, 0, &err); // bpv 0
+    Assert( err == GRIB_ENCODING_ERROR);
+    Assert( result == 0 );
+}
+
+static void test_filepool()
+{
+    printf("Running %s ...\n", __func__);
+    grib_file_pool_print("file_pool contents", stdout);
+}
+
 int main(int argc, char** argv)
 {
     printf("Doing unit tests. ecCodes version = %ld\n", grib_get_api_version());
+    printf("codes_print_api_version gives: ");
+    codes_print_api_version(stdout);
+    printf("\n");
+
+    test_codes_context_set_debug();
+    test_codes_get_error_message();
+    test_grib_get_binary_scale_fact();
 
     test_iarray();
     test_darray();
@@ -636,13 +881,19 @@ int main(int argc, char** argv)
 
     test_scale_factor_scaled_values();
     test_dates();
+    test_time_conversions();
     test_logging_proc();
+    test_print_proc();
     test_grib_binary_search();
     test_parse_keyval_string();
 
     test_get_git_sha1();
+    test_get_package_name();
     test_get_build_date();
     test_gribex_mode();
+    test_gts_header_mode();
+    test_bufr_multi_element_constant_arrays();
+    test_data_quality_checks();
 
     test_concept_condition_strings();
 
@@ -675,5 +926,11 @@ int main(int argc, char** argv)
     test_string_replace_char();
     test_string_remove_char();
 
+    test_grib2_select_PDTN();
+    test_grib2_choose_PDTN();
+    test_codes_is_feature_enabled();
+    test_codes_get_features();
+    test_filepool();
+
     return 0;
 }
